# coding: utf-8

metadata = dict(
  __name__     = "pandoc",
  __version__  = "1.0.0b2", 
  __license__  = "MIT License",
  __author__   = u"Sébastien Boisgérault <Sebastien.Boisgerault@mines-paristech.fr>",
  __url__      = "https://github.com/boisgera/pandoc",
  __summary__  = "Pandoc Documents for Python",
  __keywords__ = """
                 software development, topic / text processing, 
                 python / 2, python / 3,
                 beta
                 """
)

globals().update(metadata)
__all__ = list(metadata.keys())

